/*
 * Sixth - System for data storage, computation, exploration and interaction.
 * Copyright ©2012-2016, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.sixth.e3d.examples;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;
import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.ViewFrame;
import eu.svjatoslav.sixth.e3d.renderer.raster.ShapeCollection;
import eu.svjatoslav.sixth.e3d.renderer.raster.shapes.composite.Graph;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class GraphDemo {

    private static final double scale = 50d;

    private static Graph getCosinusGraph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            final double y = Math.cos(x);

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "Cosinus", location);
    }

    private static Graph getFormula1Graph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            final double y = Math.sin(Math.tan(x));

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "y = sin(tan(x))", location);
    }

    private static Graph getFormula2Graph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            final double y = (Math.pow((10 - x), 2) / 30) - 2;

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "y = ( (10-x)^2 ) / 30", location);
    }

    private static Graph getFormula3Graph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            final double y = Math.sin(x / 2) + Math.sin(x / 1.26);

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "y = sin(x/2) + sin(x/1.26)", location);
    }

    private static Graph getSinusGraph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            final double y = Math.sin(x);

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "Sinus", location);
    }

    private static Graph getTangentGraph(final Point3D location)
            throws IOException {
        final List<Point2D> data = new ArrayList<>();
        for (double x = 0; x < 20; x += 0.25) {
            double y = Math.tan(x);

            if (y > 2)
                y = 2;
            if (y < -2)
                y = -2;

            final Point2D p = new Point2D(x, y);
            data.add(p);
        }

        return new Graph(scale, data, "Tangent", location);
    }

    public static void main(final String[] args) throws IOException {

        final ViewFrame viewFrame = new ViewFrame();

        final ShapeCollection geometryCollection = viewFrame.getView()
                .getContext().getRootShapeCollection();

        Point3D location = new Point3D(-600, -300, 0);
        geometryCollection.addShape(getSinusGraph(location));

        location = new Point3D(600, -300, 0);
        geometryCollection.addShape(getFormula1Graph(location));

        location = new Point3D(-600, 0, 0);
        geometryCollection.addShape(getCosinusGraph(location));

        location = new Point3D(600, 0, 0);
        geometryCollection.addShape(getFormula2Graph(location));

        location = new Point3D(-600, 300, 0);
        geometryCollection.addShape(getTangentGraph(location));

        location = new Point3D(600, 300, 0);
        geometryCollection.addShape(getFormula3Graph(location));

        viewFrame.getView().getContext().getAvatar()
                .setLocation(new Point3D(0, 0, -500));

    }
}
